/*
 * Copyright (C) 2001-2013 Tommy Scheunemann <net@arrishq.net>
 *
 * Modified: 2004/08/03 Romildo <romildo@uber.com.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Functions needed for producing a enlightenment D16 directory hierarchy
 * configuration
 *
 * In this case the rootmenu is the top level directory
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <pwd.h>
#include <strings.h>
#include <string.h>

#include "wmconfig.h"
#include "prototypes.h"
#include "package.h"

#define OUTPUTDIR ".enlightenment/wmconfig"

#if (defined(__unix__) || defined(unix)) && !defined(USG)
#include <limits.h>
#include <sys/param.h>
#endif

extern const char *root_menu_name;
extern const char *output_directory;
extern unsigned int flags;

static void output_e16_level(struct group *root, FILE *f, char *dir_name,const char *use_term)
{
    struct item *item;
    char file[PATH_MAX];
    int fd;
    FILE *f_tmp;

    if (root == (struct group *)NULL)
	return;

    item = root->items;
    while (item->type != 0) {
	if (item->type == ITEM_MENU) {
	    struct group *g;
	    g = (struct group *)item->data;

	    fprintf(f, "\"%s\" NULL menu \"%s/%s.menu\"\n", g->name, dir_name, g->name);
	    f_tmp = f;
	    snprintf(file, sizeof(file), "%s/%s.menu", dir_name, g->name);

#if defined(BSD)
	    fd = open(file, O_CREAT | O_EXCL | O_NOFOLLOW | O_RDWR, 0600);
#else
	    fd = open(file, O_CREAT | O_EXCL | O_RDWR, 0600);
#endif

	    f = fdopen (fd, "wx");
	    if (f == (FILE *) NULL) {
	        fprintf(stderr, gettext ("Could not create file %s\n"), file);
		perror("Error code");
		return;
	    }
	    fprintf(f, "\"%s\"\n", g->name);
	    output_e16_level(item->data, f, dir_name, use_term);
	    fclose(f);
	    f = f_tmp;

	} else if (item->type == ITEM_APP) {

	    struct package *app;
	    app = (struct package *)item->data;
	    if (app->restart) {

		// Restart function requested, run by calling eesh
		if (strncasecmp(app->restart, "restart", 7) == 0)
		    fprintf(f, " \"%s\" \"%s\" exec \"eesh -e 'restart'\"\n", app->name, app->mini_icon == NULL || is_set(NO_MINI_ICONS) ? "NULL" : app->mini_icon);

		// Quit function requested, run by calling eesh
		else if (strncasecmp(app->restart, "quit", 4) == 0)
		    fprintf(f, " \"%s\" \"%s\" exec \"eesh -e 'exit'\"\n", app->name, app->mini_icon == NULL || is_set(NO_MINI_ICONS) ? "NULL" : app->mini_icon);

		// Otherwise do a restart, again by calling eesh
		else
		    fprintf(f, " \"%s\" \"%s\" exec \"eesh -e 'restart_wm %s'\"\n", app->name, app->mini_icon == NULL || is_set(NO_MINI_ICONS) ? "NULL" : app->mini_icon, app->exec);
	    } else {
		if (app->name && app->exec) {
		    if (app->terminal)
			fprintf(f, "\"%s\" %s exec \"%s -e %s\"\n", app->name, app->mini_icon == NULL || is_set(NO_MINI_ICONS) ? "NULL" : app->mini_icon, use_term, app->exec);
		    else
			fprintf(f, "\"%s\" %s exec \"%s\"\n", app->name, app->mini_icon == NULL || is_set(NO_MINI_ICONS) ? "NULL" : app->mini_icon, app->exec);
	        }
	    }
	}
	item++;
    }
}

static void clean_dir(const char *dir_name)
{
    struct dirent **namelist;
    int nr_files;
    int i;
    
    if (dir_name == NULL)
	return;
    
    /* read the whole directory in one pass */
    nr_files = scandir(dir_name, &namelist, NULL, alphasort);
    for (i = 0; i < nr_files; i++) {
	if ( (strcmp(namelist[i]->d_name, ".") != 0) &&
	     (strcmp(namelist[i]->d_name, "..") != 0) ) {
	    struct stat st;
	    char tmp[PATH_MAX];
	    int retval;
	    
	    snprintf(tmp, sizeof(tmp), "%s/%s", dir_name, namelist[i]->d_name);
	    if (stat(tmp, &st) != 0) {
		fprintf(stderr, gettext ("Could not stat %s\n"), tmp);
		return;
	    }

	    if (S_ISDIR(st.st_mode)) {
		clean_dir(tmp);
		retval = rmdir(tmp);

	    } else {

		retval = unlink(tmp);
	    }

	    if (retval != 0) {
		perror("Could not remove");
		return;
	    }
	}

	free(namelist[i]);
    }
    if (nr_files && namelist)
	free(namelist);	    
}


void output_enlightenment(struct group *root,const char *use_term)
{
    struct stat st;
    char root_dir[PATH_MAX];
    char file[PATH_MAX];
    struct passwd *pw;
    int fd;
    FILE * f;
    
    setpwent();
    pw = getpwuid(getuid());
    if (pw == (struct passwd *)NULL) {
	fprintf(stderr, gettext ("Could not find out who you are (getpwnam failed)!\n"));
	return;
    }
    endpwent();

    if (output_directory == NULL) {
     snprintf(root_dir, sizeof(root_dir), "%s/%s", pw->pw_dir, OUTPUTDIR);
    } else {
     snprintf(root_dir, sizeof(root_dir), "%s", output_directory);
    }

    /* First, check if the root_dir is a dir */
    if (stat(root_dir, &st) == 0) {
	/* that one exists */
	if (!S_ISDIR(st.st_mode)) {
	    fprintf(stderr, gettext ("Error: %s exists but it is not a directory\n"), root_dir);
	    return;
	}
	clean_dir(root_dir);
    }  else {
	/* we must make that directory first */
	if (mkdir(root_dir, 0700) != 0) {
	    fprintf(stderr, gettext ("Could not create directory '%s'\n"), root_dir);
	    return;
	}
    }
    snprintf(file, sizeof(file), "%s/%s.menu", root_dir, root_menu_name);

#if defined(BSD)
    fd = open(file, O_CREAT | O_EXCL | O_NOFOLLOW | O_RDWR, 0600);
#else
    fd = open(file, O_CREAT | O_EXCL | O_RDWR, 0600);
#endif

    f = fdopen (fd, "wx");
    if (f == (FILE *) NULL) {
        fprintf(stderr, gettext ("Could not create file %s\n"), file);
	perror("Error code");
	return;
    }
    fprintf(f, "\"%s Menu\"\n", root_menu_name);
    output_e16_level(root, f, root_dir, use_term);
    fclose(f);
}
