/*
 * Copyright (C) 2004-2013 Tommy Scheunemann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Functions needed for producing a Wmx / Flwm directory hierarchy
 * configuration
 *
 * In this case the rootmenu is the top level directory
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <pwd.h>
#include <strings.h>

#include "wmconfig.h"
#include "prototypes.h"
#include "package.h"

#if (defined(__unix__) || defined(unix)) && !defined(USG)
#include <limits.h>
#include <sys/param.h>
#endif

#define WMX_DIR		".wmx/wmconfig"

extern const char *root_menu_name;

static void create_launcher(const char *file, struct package *app,const char *use_term)
{
    int fd;
    FILE *f;
    if (app == (struct package *)NULL) {
	return;
    }

    /* Remove the & */
    if (app->exec != NULL) {
	char *p;
	p = strrchr(app->exec, '&');
	if (p != NULL)
	    *p = '\0';
    }

    /*  At the very least we need a filename and command to execute:  */
    if (file == NULL || app->exec == NULL || app->restart)
	return;

#if defined(BSD)
    fd = open(file, O_CREAT | O_EXCL | O_NOFOLLOW | O_RDWR, 0600);
#else
    fd = open(file, O_CREAT | O_EXCL | O_RDWR, 0600);
#endif

    f = fdopen (fd, "wx");
    if (f == (FILE *) NULL) {
	fprintf(stderr, gettext ("Could not create file %s\n"), file);
	perror("Error code");
	return;
    }

    /*  Write the file:  */
    fprintf(f, "#!/bin/sh\n");
    if (app->terminal != NULL)
    {
	fprintf(f, "exec %s -e %s\n", use_term, app->exec);
    } else {
	fprintf(f, "exec %s\n", app->exec);
    }
    fclose(f);
    fchmod (fd, S_IRWXU);
}

/*  Like mkdir -p  */
static void make_directory_path(const char *path)
{
    char buf[PATH_MAX];
    int pi=0;
    int bi=0;
    while( path[pi] != '\0' ) {
	buf[bi] = path[pi];
	bi++ ; pi++ ;
	buf[bi] = '\0';
	if ( path[pi] == '\0' || path[pi] == '/' ) {
	    mkdir(buf, 0700);
	}
    }
}

static void make_dir(struct group *root, int level, const char *dir_name,const char *use_term)
{
    struct item *item;
    char c_tmp[PATH_MAX];

    if (root == (struct group *)NULL)
	return;

    item = root->items;
    while (item->type != 0) {
	if (item->type == ITEM_MENU) {
	    struct group *tmp;
	    tmp = (struct group *)item->data;
	    snprintf(c_tmp, sizeof(c_tmp), "%s/%s", single_string(dir_name), single_string(tmp->name) );
	    make_directory_path(c_tmp);
	} else if (item->type == ITEM_APP) {
	    struct package *app;
	    app = (struct package *)item->data;
	    if (app->name && app->exec) {
		snprintf(c_tmp, sizeof(c_tmp), "%s/%s", single_string(dir_name), single_string(app->name) );
		create_launcher(c_tmp, app, use_term);
	    }
	}
	item++;
    }

    /* second pass, recursive output... */
    item = root->items;
    while (item->type != 0) {
	if (item->type == ITEM_MENU) {
            struct group *tmp;
	    tmp = (struct group *)item->data;
	    snprintf(c_tmp, sizeof(c_tmp), "%s/%s", dir_name, tmp->name);
	    make_dir(item->data, level+1, c_tmp, use_term);
	}
	item++;
    }
}

static void clean_dir(const char *dir_name)
{
    struct dirent **namelist;
    int nr_files;
    int i;

    if (dir_name == NULL)
	return;

    /* read the whole directory in one pass */
    nr_files = scandir(dir_name, &namelist, NULL, alphasort);
    for (i = 0; i < nr_files; i++) {
	if ( (strcmp(namelist[i]->d_name, ".") != 0) && (strcmp(namelist[i]->d_name, "..") != 0) ) {
	    struct stat st;
	    char tmp[PATH_MAX];
	    int retval;

	    snprintf(tmp, sizeof(tmp), "%s/%s", dir_name, namelist[i]->d_name);
	    if (stat(tmp, &st) != 0) {
		fprintf(stderr, gettext ("Could not stat %s\n"), tmp);
		return;
	    }
	    if (S_ISDIR(st.st_mode)) {
		clean_dir(tmp);
		retval = rmdir(tmp);
	    } else {
		retval = unlink(tmp);
	    }
	    if (retval != 0) {
		perror("Could not remove");
		return;
	    }
	}
	free(namelist[i]);
    }
    if (nr_files && namelist)
	free(namelist);	    
}

// Main function
void output_wmx(struct group *root,const char *use_term)
{
    struct stat st;
    char root_dir[PATH_MAX];
    struct passwd *pw;
    
    setpwent();
    pw = getpwuid(getuid());
    if (pw == (struct passwd *)NULL) {
	fprintf(stderr, gettext ("Could not find out who you are (getpwnam failed)!\n"));
	return;
    }
    endpwent();
    snprintf(root_dir, sizeof(root_dir), "%s/%s", pw->pw_dir, WMX_DIR);
   
    // First, check if the root_dir already exists and remove it
    if (stat(root_dir, &st) == 0) {
	// that one exists
	if (!S_ISDIR(st.st_mode)) {
	    fprintf(stderr, gettext ("Error: %s exists but it is not a directory\n"),
		    root_dir);
	    return;
	}
	clean_dir(root_dir);
	rmdir(root_dir);
    }

    /* create_wmx_menu_directory(root_dir, DEFAULT_ROOT_NAME); */
    make_dir(root, 0, root_dir, use_term);
}
